@@ -1,5 +1,5 @@
 /* Obsolete function to get current working directory.
-   Copyright (C) 1991-2014 Free Software Foundation, Inc.
+   Copyright (C) 1991-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <errno.h>
 #include <limits.h>
@@ -23,8 +23,7 @@
 
 
 char *
-getwd (buf)
-     char *buf;
+getwd (char *buf)
 {
 #ifndef PATH_MAX
 #define PATH_MAX 1024
