@@ -1,7 +1,6 @@
 /* Duplicate handle for selection of locales.
-   Copyright (C) 1997-2017 Free Software Foundation, Inc.
+   Copyright (C) 1997-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1997.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -15,7 +14,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <locale.h>
 #include <libc-lock.h>
@@ -29,8 +28,8 @@
 __libc_rwlock_define (extern , __libc_setlocale_lock attribute_hidden)
 
 
-__locale_t
-__duplocale (__locale_t dataset)
+locale_t
+__duplocale (locale_t dataset)
 {
   /* This static object is returned for newlocale (LC_ALL_MASK, "C").  */
   if (dataset == _nl_C_locobj_ptr)
@@ -40,10 +39,15 @@ __duplocale (__locale_t dataset)
   if (dataset == LC_GLOBAL_LOCALE)
     dataset = &_nl_global_locale;
 
-  __locale_t result;
+  locale_t result;
   int cnt;
   size_t names_len = 0;
 
+  /* If dataset points to _nl_global_locale, we need to prevent other
+     threads from modifying it.  We also modify global data below (the
+     usage counts).  */
+  __libc_rwlock_wrlock (__libc_setlocale_lock);
+
   /* Calculate the total space we need to store all the names.  */
   for (cnt = 0; cnt < __LC_LAST; ++cnt)
     if (cnt != LC_ALL && dataset->__names[cnt] != _nl_C_name)
@@ -56,9 +60,6 @@ __duplocale (__locale_t dataset)
     {
       char *namep = (char *) (result + 1);
 
-      /* We modify global data (the usage counts).  */
-      __libc_rwlock_wrlock (__libc_setlocale_lock);
-
       for (cnt = 0; cnt < __LC_LAST; ++cnt)
 	if (cnt != LC_ALL)
 	  {
@@ -79,11 +80,11 @@ __duplocale (__locale_t dataset)
       result->__ctype_b = dataset->__ctype_b;
       result->__ctype_tolower = dataset->__ctype_tolower;
       result->__ctype_toupper = dataset->__ctype_toupper;
-
-      /* It's done.  */
-      __libc_rwlock_unlock (__libc_setlocale_lock);
     }
 
+  /* It's done.  */
+  __libc_rwlock_unlock (__libc_setlocale_lock);
+
   return result;
 }
 weak_alias (__duplocale, duplocale)
