@@ -1,6 +1,5 @@
-/* Copyright (C) 1995-2021 Free Software Foundation, Inc.
+/* Copyright (C) 1995-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Ulrich Drepper <drepper@gnu.org>, 1995.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
@@ -220,10 +219,8 @@ No definition for %s category found"), "LC_TIME");
 	}
       else
 	{
-	  static const uint32_t wt_fmt_ampm[]
-	    = { '%','I',':','%','M',':','%','S',' ','%','p',0 };
 	  time->t_fmt_ampm = "%I:%M:%S %p";
-	  time->wt_fmt_ampm = wt_fmt_ampm;
+	  time->wt_fmt_ampm = (const uint32_t *) L"%I:%M:%S %p";
 	}
     }
 
@@ -233,7 +230,7 @@ No definition for %s category found"), "LC_TIME");
       const int days_per_month[12] = { 31, 29, 31, 30, 31, 30,
 				       31, 31, 30, 31 ,30, 31 };
       size_t idx;
-      uint32_t *wstr;
+      wchar_t *wstr;
 
       time->era_entries =
 	(struct era_data *) xmalloc (time->num_era
@@ -459,18 +456,18 @@ No definition for %s category found"), "LC_TIME");
 	    }
 
 	  /* Now generate the wide character name and format.  */
-	  wstr = wcschr_uint32 (time->wera[idx], L':'); /* end direction */
-	  wstr = wstr ? wcschr_uint32 (wstr + 1, L':') : NULL; /* end offset */
-	  wstr = wstr ? wcschr_uint32 (wstr + 1, L':') : NULL; /* end start */
-	  wstr = wstr ? wcschr_uint32 (wstr + 1, L':') : NULL; /* end end */
+	  wstr = wcschr ((wchar_t *) time->wera[idx], L':');/* end direction */
+	  wstr = wstr ? wcschr (wstr + 1, L':') : NULL;	/* end offset */
+	  wstr = wstr ? wcschr (wstr + 1, L':') : NULL;	/* end start */
+	  wstr = wstr ? wcschr (wstr + 1, L':') : NULL;	/* end end */
 	  if (wstr != NULL)
 	    {
-	      time->era_entries[idx].wname = wstr + 1;
-	      wstr = wcschr_uint32 (wstr + 1, L':'); /* end name */
+	      time->era_entries[idx].wname = (uint32_t *) wstr + 1;
+	      wstr = wcschr (wstr + 1, L':');	/* end name */
 	      if (wstr != NULL)
 		{
 		  *wstr = L'\0';
-		  time->era_entries[idx].wformat = wstr + 1;
+		  time->era_entries[idx].wformat = (uint32_t *) wstr + 1;
 		}
 	      else
 		time->era_entries[idx].wname =
@@ -529,16 +526,7 @@ No definition for %s category found"), "LC_TIME");
   if (time->date_fmt == NULL)
     time->date_fmt = "%a %b %e %H:%M:%S %Z %Y";
   if (time->wdate_fmt == NULL)
-    {
-      static const uint32_t wdate_fmt[] =
-	{ '%','a',' ',
-	  '%','b',' ',
-	  '%','e',' ',
-	  '%','H',':','%','M',':','%','S',' ',
-	  '%','Z',' ',
-	  '%','Y',0 };
-      time->wdate_fmt = wdate_fmt;
-    }
+    time->wdate_fmt = (const uint32_t *) L"%a %b %e %H:%M:%S %Z %Y";
 }
 
 
