@@ -1,6 +1,5 @@
-/* Copyright (C) 2005-2014 Free Software Foundation, Inc.
+/* Copyright (C) 2005-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Richard Henderson <rth@redhat.com>, 2005.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -14,7 +13,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <stdbool.h>
 #include <math.h>
@@ -38,35 +37,35 @@ __multc3 (long double a, long double b, long double c, long double d)
     {
       /* Recover infinities that computed as NaN + iNaN.  */
       bool recalc = 0;
-      if (__isinf_nsl (a) || __isinf_nsl (b))
+      if (isinf (a) || isinf (b))
 	{
 	  /* z is infinite.  "Box" the infinity and change NaNs in
 	     the other factor to 0.  */
-	  a = __copysignl (__isinf_nsl (a) ? 1 : 0, a);
-	  b = __copysignl (__isinf_nsl (b) ? 1 : 0, b);
-	  if (isnan (c)) c = __copysignl (0, c);
-	  if (isnan (d)) d = __copysignl (0, d);
+	  a = copysignl (isinf (a) ? 1 : 0, a);
+	  b = copysignl (isinf (b) ? 1 : 0, b);
+	  if (isnan (c)) c = copysignl (0, c);
+	  if (isnan (d)) d = copysignl (0, d);
 	  recalc = 1;
 	}
-     if (__isinf_nsl (c) || __isinf_nsl (d))
+     if (isinf (c) || isinf (d))
 	{
 	  /* w is infinite.  "Box" the infinity and change NaNs in
 	     the other factor to 0.  */
-	  c = __copysignl (__isinf_nsl (c) ? 1 : 0, c);
-	  d = __copysignl (__isinf_nsl (d) ? 1 : 0, d);
-	  if (isnan (a)) a = __copysignl (0, a);
-	  if (isnan (b)) b = __copysignl (0, b);
+	  c = copysignl (isinf (c) ? 1 : 0, c);
+	  d = copysignl (isinf (d) ? 1 : 0, d);
+	  if (isnan (a)) a = copysignl (0, a);
+	  if (isnan (b)) b = copysignl (0, b);
 	  recalc = 1;
 	}
      if (!recalc
-	  && (__isinf_nsl (ac) || __isinf_nsl (bd)
-	      || __isinf_nsl (ad) || __isinf_nsl (bc)))
+	  && (isinf (ac) || isinf (bd)
+	      || isinf (ad) || isinf (bc)))
 	{
 	  /* Recover infinities from overflow by changing NaNs to 0.  */
-	  if (isnan (a)) a = __copysignl (0, a);
-	  if (isnan (b)) b = __copysignl (0, b);
-	  if (isnan (c)) c = __copysignl (0, c);
-	  if (isnan (d)) d = __copysignl (0, d);
+	  if (isnan (a)) a = copysignl (0, a);
+	  if (isnan (b)) b = copysignl (0, b);
+	  if (isnan (c)) c = copysignl (0, c);
+	  if (isnan (d)) d = copysignl (0, d);
 	  recalc = 1;
 	}
       if (recalc)
