@@ -1,5 +1,5 @@
 /* Test for completion signal handling.
-   Copyright (C) 2000-2017 Free Software Foundation, Inc.
+   Copyright (C) 2000-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <aio.h>
 #include <signal.h>
@@ -23,9 +23,6 @@
 #include <unistd.h>
 #include <errno.h>
 
-/* We might need a bit longer timeout.  */
-#define TIMEOUT 10 /* sec */
-
 int my_signo;
 
 volatile sig_atomic_t flag;
