@@ -1,4 +1,4 @@
-/* Copyright (C) 1991-2018 Free Software Foundation, Inc.
+/* Copyright (C) 1991-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <stddef.h>
 #include <stdio.h>
@@ -25,7 +25,7 @@
 ssize_t
 __getline (char **lineptr, size_t *n, FILE *stream)
 {
-  return _IO_getdelim (lineptr, n, '\n', stream);
+  return __getdelim (lineptr, n, '\n', stream);
 }
 
 weak_alias (__getline, getline)
