@@ -1,4 +1,4 @@
-/* Test scanf functions with C2X binary integers (narrow strings,
+/* Test scanf functions with C23 binary integers (narrow strings,
    GNU extensions to C89).
    Copyright (C) 2022-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
@@ -20,7 +20,7 @@
 #define CHAR char
 #define FNW
 #define L_(C) C
-#define TEST_C2X 0
+#define TEST_C23 0
 #define STD "gnu89"
 
 #include <tst-scanf-binary-main.c>
