@@ -1,5 +1,5 @@
 /* Convert string representing a number to integer value, using given locale.
-   Copyright (C) 1997-2022 Free Software Foundation, Inc.
+   Copyright (C) 1997-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -18,9 +18,13 @@
 
 #define QUAD	1
 
+#include <features.h>
+#undef __GLIBC_USE_C23_STRTOL
+#define __GLIBC_USE_C23_STRTOL 0
 #include <locale.h>
+#include <stdbool.h>
 
 extern long long int ____strtoll_l_internal (const char *, char **, int, int,
-					     locale_t);
+					     bool, locale_t);
 
 #include <strtol_l.c>
