@@ -1,6 +1,13 @@
-/* Test case by Joseph S. Myers <jsm28@cam.ac.uk>.  */
 #undef __USE_STRING_INLINES
 #define __USE_STRING_INLINES
+#include <sys/cdefs.h>
+#include <libc-diag.h>
+#if __GNUC_PREREQ (8, 0)
+/* GCC warns about strncat truncating output; this is deliberately
+   tested here.  If fortify is enabled, it is also triggered by the
+   wrappers. */
+DIAG_IGNORE_NEEDS_COMMENT (8, "-Wstringop-truncation");
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -10,7 +17,9 @@ char d[3] = "\0\1\2";
 int
 main (void)
 {
+  DIAG_PUSH_NEEDS_COMMENT;
   strncat (d, "\5\6", 1);
+  DIAG_POP_NEEDS_COMMENT;
   if (d[0] != '\5')
     {
       puts ("d[0] != '\\5'");
@@ -23,9 +32,9 @@ main (void)
     }
   if (d[2] != '\2')
     {
-      puts ("d[2] != '\\3'");
+      puts ("d[2] != '\\2'");
       exit (1);
     }
 
-  exit (0);
+  return 0;
 }
