@@ -1,6 +1,5 @@
-/* Copyright (C) 1999-2017 Free Software Foundation, Inc.
+/* Copyright (C) 2022-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Andreas Jaeger <aj@suse.de>, 1999.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -14,12 +13,14 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #ifndef _LDCONFIG_H
 #define _LDCONFIG_H
 
+#include <stddef.h>
 #include <stdint.h>
+#include <sys/stat.h>
 
 #define FLAG_ANY			-1
 #define FLAG_TYPE_MASK			0x00ff
@@ -29,7 +30,6 @@
 #define FLAG_ELF_LIBC6			0x0003
 #define FLAG_REQUIRED_MASK		0xff00
 #define FLAG_SPARC_LIB64		0x0100
-#define FLAG_IA64_LIB64			0x0200
 #define FLAG_X8664_LIB64		0x0300
 #define FLAG_S390_LIB64			0x0400
 #define FLAG_POWERPC_LIB64		0x0500
@@ -42,6 +42,10 @@
 #define FLAG_MIPS_LIB32_NAN2008		0x0c00
 #define FLAG_MIPS64_LIBN32_NAN2008	0x0d00
 #define FLAG_MIPS64_LIBN64_NAN2008	0x0e00
+#define FLAG_RISCV_FLOAT_ABI_SOFT	0x0f00
+#define FLAG_RISCV_FLOAT_ABI_DOUBLE	0x1000
+#define FLAG_LARCH_FLOAT_ABI_SOFT	0x1100
+#define FLAG_LARCH_FLOAT_ABI_DOUBLE	0x1200
 
 /* Name of auxiliary cache.  */
 #define _PATH_LDCONFIG_AUX_CACHE "/var/cache/ldconfig/aux-cache"
@@ -53,31 +57,43 @@ extern void init_cache (void);
 
 extern void save_cache (const char *cache_name);
 
-extern void add_to_cache (const char *path, const char *lib, int flags,
-			  unsigned int osversion, uint64_t hwcap);
+struct glibc_hwcaps_subdirectory;
+
+/* Return a struct describing the subdirectory for NAME.  Reuse an
+   existing struct if it exists.  */
+struct glibc_hwcaps_subdirectory *new_glibc_hwcaps_subdirectory
+  (const char *name);
+
+/* Returns the name that was specified when
+   add_glibc_hwcaps_subdirectory was called.  */
+const char *glibc_hwcaps_subdirectory_name
+  (const struct glibc_hwcaps_subdirectory *);
+
+extern void add_to_cache (const char *path, const char *filename,
+			  const char *soname, int flags,
+			  unsigned int isa_level,
+			  struct glibc_hwcaps_subdirectory *);
 
 extern void init_aux_cache (void);
 
 extern void load_aux_cache (const char *aux_cache_name);
 
-extern int search_aux_cache (struct stat64 *stat_buf, int *flags,
-			     unsigned int *osversion, char **soname);
+extern int search_aux_cache (struct stat *stat_buf, int *flags,
+			     unsigned int *isa_level, char **soname);
 
-extern void add_to_aux_cache (struct stat64 *stat_buf, int flags,
-			      unsigned int osversion, const char *soname);
+extern void add_to_aux_cache (struct stat *stat_buf, int flags,
+			      unsigned int isa_level, const char *soname);
 
 extern void save_aux_cache (const char *aux_cache_name);
 
 /* Declared in readlib.c.  */
 extern int process_file (const char *real_file_name, const char *file_name,
-			 const char *lib, int *flag, unsigned int *osversion,
-			 char **soname, int is_link, struct stat64 *stat_buf);
-
-extern char *implicit_soname (const char *lib, int flag);
+			 const char *lib, int *flag, unsigned int *isa_level,
+			 char **soname, int is_link, struct stat *stat_buf);
 
 /* Declared in readelflib.c.  */
-extern int process_elf_file (const char *file_name, const char *lib, int *flag,
-			     unsigned int *osversion, char **soname,
+extern int process_elf_file (const char *file_name, const char *lib,
+			     int *flag, unsigned int *isa_level, char **soname,
 			     void *file_contents, size_t file_length);
 
 /* Declared in chroot_canon.c.  */
@@ -86,7 +102,14 @@ extern char *chroot_canon (const char *chroot, const char *name);
 /* Declared in ldconfig.c.  */
 extern int opt_verbose;
 
-extern int opt_format;
+enum opt_format
+  {
+    opt_format_old = 0,	/* Use struct cache_file.  */
+    opt_format_compat = 1, /* Use both, old format followed by new.  */
+    opt_format_new = 2,	/* Use struct cache_file_new.  */
+  };
+
+extern enum opt_format opt_format;
 
 /* Prototypes for a few program-wide used functions.  */
 #include <programs/xmalloc.h>
