@@ -1,5 +1,5 @@
 /* Multiply long double (ldbl-128) values, narrowing the result to double.
-   Copyright (C) 2018-2019 Free Software Foundation, Inc.
+   Copyright (C) 2018-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #define f32xmulf64x __hide_f32xmulf64x
 #define f32xmulf128 __hide_f32xmulf128
@@ -32,6 +32,6 @@ double
 __dmull (_Float128 x, _Float128 y)
 {
   NARROW_MUL_ROUND_TO_ODD (x, y, double, union ieee854_long_double, l,
-			   mantissa3);
+			   mantissa3, false);
 }
 libm_alias_double_ldouble (mul)
