@@ -1,5 +1,5 @@
 /* Compatibility routine for IEEE double as long double for totalordermag.
-   Copyright (C) 2016-2019 Free Software Foundation, Inc.
+   Copyright (C) 2016-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -14,13 +14,13 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include "nldbl-compat.h"
 
 double
 attribute_hidden
-totalordermagl (double x, double y)
+totalordermagl (const double *x, const double *y)
 {
   return totalordermag (x, y);
 }
