@@ -1,5 +1,5 @@
 /* getresuid -- fetch real user ID, effective user ID, and saved-set user ID
-   Copyright (C) 2002-2017 Free Software Foundation, Inc.
+   Copyright (C) 2002-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <errno.h>
 #include <unistd.h>
@@ -27,8 +27,8 @@ int
 __getresuid (uid_t *ruid, uid_t *euid, uid_t *suid)
 {
   error_t err;
-  uid_t real, eff, saved;
 
+retry:
   HURD_CRITICAL_BEGIN;
   __mutex_lock (&_hurd_id.lock);
 
@@ -40,22 +40,21 @@ __getresuid (uid_t *ruid, uid_t *euid, uid_t *suid)
 	err = EGRATUITOUS;
       else
 	{
-	  real = _hurd_id.aux.uids[0];
-	  eff = _hurd_id.gen.nuids < 1 ? real : _hurd_id.gen.uids[0];
-	  saved = _hurd_id.aux.nuids < 2 ? real : _hurd_id.aux.uids[1];
+	  uid_t real = _hurd_id.aux.uids[0];
+
+	  *ruid = real;
+	  *euid = _hurd_id.gen.nuids < 1 ? real : _hurd_id.gen.uids[0];
+	  *suid = _hurd_id.aux.nuids < 2 ? real : _hurd_id.aux.uids[1];
 	}
     }
 
   __mutex_unlock (&_hurd_id.lock);
   HURD_CRITICAL_END;
+  if (err == EINTR)
+    /* Got a signal while inside an RPC of the critical section, retry again */
+    goto retry;
 
-  if (err)
-    return __hurd_fail (err);
-
-  *ruid = real;
-  *euid = eff;
-  *suid = saved;
-  return 0;
+  return __hurd_fail (err);
 }
 libc_hidden_def (__getresuid)
 weak_alias (__getresuid, getresuid)
