@@ -1,4 +1,4 @@
-/* Copyright (C) 1992,94,97,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1992-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -12,9 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
-   License along with the GNU C Library; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307 USA.  */
+   License along with the GNU C Library; if not, see
+   <https://www.gnu.org/licenses/>.  */
 
 #include <errno.h>
 #include <sys/socket.h>
@@ -29,12 +28,11 @@
 
 /* XXX should be __getsockopt ? */
 int
-getsockopt (fd, level, optname, optval, optlen)
-     int fd;
-     int level;
-     int optname;
-     void *optval;
-     size_t *optlen;
+getsockopt (int fd,
+	    int level,
+	    int optname,
+	    void *optval,
+	    socklen_t *optlen)
 {
   error_t err;
   char *buf = optval;
