@@ -1,4 +1,4 @@
-/* Copyright (C) 1992-2023 Free Software Foundation, Inc.
+/* Copyright (C) 1992-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -42,8 +42,7 @@ __sigaltstack (const stack_t *argss, stack_t *oss)
     {
       /* Can't disable a stack that is in use.  */
       __spin_unlock (&s->lock);
-      errno = EINVAL;
-      return -1;
+      return __hurd_fail (EINVAL);
     }
 
   old = s->sigaltstack;
