@@ -1,5 +1,5 @@
 /* Generic definitions for dealing with Mach thread states.
-   Copyright (C) 1994-2016 Free Software Foundation, Inc.
+   Copyright (C) 1994-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 
 /* Everything else is called `thread_state', but CMU's header file is
@@ -38,13 +38,28 @@
 #endif
 #endif
 
+/* Set up the thread state to call the given function on the given state.
+   Dependning on architecture, this may imply more than just setting PC
+   and SP.  */
+#ifndef MACHINE_THREAD_STATE_SETUP_CALL
+#define MACHINE_THREAD_STATE_SETUP_CALL(ts, stack, size, func) \
+  (MACHINE_THREAD_STATE_SET_PC (ts, func), \
+   MACHINE_THREAD_STATE_SET_SP (ts, stack, size))
+#endif
+
+/* This copies architecture-specific bits from the current thread to the new
+   thread state.  */
+#ifndef MACHINE_THREAD_STATE_FIX_NEW
+# define MACHINE_THREAD_STATE_FIX_NEW(ts)
+#endif
+
 /* These functions are of use in machine-dependent signal trampoline
    implementations.  */
 
 #include <string.h>		/* size_t, memcpy */
 #include <mach/mach_interface.h> /* __thread_get_state */
 
-static inline int
+static __inline int
 machine_get_state (thread_t thread, struct machine_thread_all_state *state,
 		   int flavor, void *stateptr, void *scpptr, size_t size)
 {
@@ -59,12 +74,12 @@ machine_get_state (thread_t thread, struct machine_thread_all_state *state,
       /* No one asked about this flavor of state before; fetch the state
 	 directly from the kernel into the sigcontext.  */
       mach_msg_type_number_t got = (size / sizeof (int));
-      return (! __thread_get_state (thread, flavor, scpptr, &got)
+      return (! __thread_get_state (thread, flavor, (thread_state_t) scpptr, &got)
 	      && got == (size / sizeof (int)));
     }
 }
 
-static inline int
+static __inline int
 machine_get_basic_state (thread_t thread,
 			 struct machine_thread_all_state *state)
 {
@@ -76,8 +91,8 @@ machine_get_basic_state (thread_t thread,
   count = MACHINE_THREAD_STATE_COUNT;
   if (__thread_get_state (thread, MACHINE_THREAD_STATE_FLAVOR,
 			  (natural_t *) &state->basic,
-			  &count) != KERN_SUCCESS ||
-      count != MACHINE_THREAD_STATE_COUNT)
+			  &count) != KERN_SUCCESS
+      || count != MACHINE_THREAD_STATE_COUNT)
     /* What kind of thread?? */
     return 0;			/* XXX */
 
