@@ -1,7 +1,6 @@
 /* Disable floating-point exceptions.
-   Copyright (C) 2000-2014 Free Software Foundation, Inc.
+   Copyright (C) 2000-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
-   Contributed by Geoffrey Keating <geoffk@geoffk.org>, 2000.
 
    The GNU C Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -15,39 +14,35 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <fenv_libc.h>
 
 int
 fedisableexcept (int excepts)
 {
-  fenv_union_t fe;
+  fenv_union_t fe, curr;
   int result, new;
 
-  result = __fegetexcept ();
+  /* Get current exception mask to return.  */
+  fe.fenv = curr.fenv = fegetenv_control ();
+  result = fenv_reg_to_exceptions (fe.l);
 
   if ((excepts & FE_ALL_INVALID) == FE_ALL_INVALID)
     excepts = (excepts | FE_INVALID) & ~ FE_ALL_INVALID;
 
-  fe.fenv = fegetenv_register ();
-  if (excepts & FE_INEXACT)
-    fe.l &= ~(1 << (31 - FPSCR_XE));
-  if (excepts & FE_DIVBYZERO)
-    fe.l &= ~(1 << (31 - FPSCR_ZE));
-  if (excepts & FE_UNDERFLOW)
-    fe.l &= ~(1 << (31 - FPSCR_UE));
-  if (excepts & FE_OVERFLOW)
-    fe.l &= ~(1 << (31 - FPSCR_OE));
-  if (excepts & FE_INVALID)
-    fe.l &= ~(1 << (31 - FPSCR_VE));
-  fesetenv_register (fe.fenv);
+  new = fenv_exceptions_to_reg (excepts);
 
-  new = __fegetexcept ();
-  if (new == 0 && result != 0)
-    (void)__fe_mask_env ();
+  if (fenv_reg_to_exceptions (new) != excepts)
+    return -1;
+
+  /* Sets the new exception mask.  */
+  fe.l &= ~new;
+
+  if (fe.l != curr.l)
+    fesetenv_control (fe.fenv);
+
+  __TEST_AND_ENTER_NON_STOP (-1ULL, fe.l);
 
-  if ((new & excepts) != 0)
-    result = -1;
   return result;
 }
