@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2021 Free Software Foundation, Inc.
+/* Copyright (C) 2003-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -27,43 +27,47 @@
 /* We can simply use the syscall.  The CPU clocks are not supported
    with this function.  */
 int
-__clock_nanosleep_time64 (clockid_t clock_id, int flags, const struct __timespec64 *req,
-                          struct __timespec64 *rem)
+__clock_nanosleep_time64 (clockid_t clock_id, int flags,
+			  const struct __timespec64 *req,
+			  struct __timespec64 *rem)
 {
   if (clock_id == CLOCK_THREAD_CPUTIME_ID)
     return EINVAL;
   if (clock_id == CLOCK_PROCESS_CPUTIME_ID)
-    clock_id = MAKE_PROCESS_CPUCLOCK (0, CPUCLOCK_SCHED);
+    clock_id = PROCESS_CLOCK;
 
   /* If the call is interrupted by a signal handler or encounters an error,
      it returns a positive value similar to errno.  */
+
 #ifndef __NR_clock_nanosleep_time64
 # define __NR_clock_nanosleep_time64 __NR_clock_nanosleep
 #endif
-  int r = INTERNAL_SYSCALL_CANCEL (clock_nanosleep_time64, clock_id,
-				   flags, req, rem);
 
-#ifndef __ASSUME_TIME64_SYSCALLS
-  if (r == 0 || r != -ENOSYS)
-    return -r;
-
-  if (! in_time_t_range (req->tv_sec))
+  int r;
+#ifdef __ASSUME_TIME64_SYSCALLS
+  r = INTERNAL_SYSCALL_CANCEL (clock_nanosleep_time64, clock_id, flags, req,
+			       rem);
+#else
+  if (!in_int32_t_range (req->tv_sec))
     {
-      __set_errno (EOVERFLOW);
-      return -1;
+      r = INTERNAL_SYSCALL_CANCEL (clock_nanosleep_time64, clock_id, flags,
+				   req, rem);
+      if (r == -ENOSYS)
+	r = -EOVERFLOW;
     }
-
-  struct timespec tr32;
-  struct timespec ts32 = valid_timespec64_to_timespec (*req);
-  r = INTERNAL_SYSCALL_CANCEL (clock_nanosleep, clock_id, flags,
-                               &ts32, &tr32);
-  if (INTERNAL_SYSCALL_ERROR_P (r))
+  else
     {
-      if (r == -EINTR && rem != NULL && (flags & TIMER_ABSTIME) == 0)
-	*rem = valid_timespec_to_timespec64 (tr32);
+      struct timespec tr32;
+      struct timespec ts32 = valid_timespec64_to_timespec (*req);
+      r = INTERNAL_SYSCALL_CANCEL (clock_nanosleep, clock_id, flags, &ts32,
+				   &tr32);
+      if (INTERNAL_SYSCALL_ERROR_P (r))
+	{
+	  if (r == -EINTR && rem != NULL && (flags & TIMER_ABSTIME) == 0)
+	    *rem = valid_timespec_to_timespec64 (tr32);
+	}
     }
-#endif /* __ASSUME_TIME64_SYSCALLS */
-
+#endif
   return -r;
 }
 
