@@ -1,4 +1,4 @@
-/* Copyright (C) 1991-2017 Free Software Foundation, Inc.
+/* Copyright (C) 1991-2024 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, see
-   <http://www.gnu.org/licenses/>.  */
+   <https://www.gnu.org/licenses/>.  */
 
 #include <errno.h>
 #include <stddef.h>
@@ -46,9 +46,9 @@ tcsetattr (int fd, int optional_actions, const struct termios *termios_p)
       return -1;
     }
 
-  if (bad_speed(termios_p->__ospeed) ||
-      bad_speed(termios_p->__ispeed == 0 ?
-		termios_p->__ospeed : termios_p->__ispeed))
+  if (bad_speed(termios_p->__ospeed)
+      || bad_speed(termios_p->__ispeed == 0
+		   ? termios_p->__ospeed : termios_p->__ispeed))
     {
       __set_errno (EINVAL);
       return -1;
